#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <m17n-gui.h>
#include <m17n-misc.h>

void
help ()
{
  printf ("Usage: findfont [FONT-SPEC]\n");
  printf ("FONT-SPEC::=[FAMILY][:script=SCRIPT][:lang=LANG][:otf=OTFSPEC]\n");
  exit (1);
}

MSymbol
make_symbol (char *str, int len)
{
  char *p = alloca (len + 1);

  memcpy (p, str, len);
  p[len] = '\0';
  return msymbol (p);
}

MFont *
parse_font_name (char *str)
{
  MFont *font = mfont ();
  char *p;

  if (*str != ':')
    {
      for (p = str + 1; *p && *p != ':'; p++);
      mfont_put_prop (font, Mfamily, make_symbol (str, p - str));
      str = p;
    }
  while (*str == ':')
    {
      str++;
      if (strncmp (str, "script=", 7) == 0)
	{
	  str += 7;
	  for (p = str; *p && *p != ':'; p++);
	  mfont_put_prop (font, Mscript, make_symbol (str, p - str));
	}
      else if (strncmp (str, "lang=", 5) == 0)
	{
	  str += 5;
	  for (p = str; *p && *p != ':'; p++);
	  mfont_put_prop (font, Mlanguage, make_symbol (str, p - str));
	}
      else if (strncmp (str, "otf=", 4) == 0)
	{
	  str += 4;
	  for (p = str; *p && *p != ':'; p++);
	  mfont_put_prop (font, msymbol ("otf"), make_symbol (str, p - str));
	}
      else
	{
	  for (p =  str; *p && *p != ':'; p++);
	  fprintf (stderr, "Unknown option ignored: %s\n",
		   msymbol_name (make_symbol (str, p - str)));
	}
      str = p;
    }
  return font;
}

void
print_font (MFont *font)
{
  MSymbol family, foundry, weight, slant, stretch;
  char *weight_name, *slant_name, *stretch_name;
  char style_name[1024];

  family = mfont_get_prop (font, Mfamily);
  foundry = mfont_get_prop (font, Mfoundry);
  weight = mfont_get_prop (font, Mweight);
  weight_name = weight ? msymbol_name (weight) : "";
  slant = mfont_get_prop (font, Mstyle);
  slant_name = slant ? msymbol_name (slant) : "";
  stretch = mfont_get_prop (font, Mstretch);
  stretch_name = stretch ? msymbol_name (stretch) : "";

  sprintf (style_name, "%s-%s-%s", weight_name, slant_name, stretch_name);
  printf ("%s:foundry=%s:style=%s\n",
	  msymbol_name (family), msymbol_name (foundry), style_name);
}

int
main (int argc, char **argv)
{
  MPlist *plist, *p;
  MFrame *frame;
  
  M17N_INIT ();

  plist = mplist ();
  mplist_add (plist, Mdevice, Mnil);
  frame = mframe (plist);
  m17n_object_unref (plist);

  if (argc == 1)
    {
      printf ("Listing all available family names...\n");
      plist = mfont_list_family_names (frame);

      for (p = plist; p && mplist_key (p) != Mnil; p = mplist_next (p))
	printf ("%s\n", msymbol_name ((MSymbol) mplist_value (p)));
    }
  else if (argc == 2)
    {
      MFont *font = parse_font_name (argv[1]);

      plist = mfont_list (frame, font, Mnil, 0);
      free (font);
      /* List up unique pairs of family and foundry of fonts in PL.  */
      for (p = plist; p && mplist_key (p) != Mnil; p = mplist_next (p))
	print_font (mplist_value (p));
    }
  else
    help ();

  if (plist)
    m17n_object_unref (plist);
  m17n_object_unref (frame);
  M17N_FINI ();
  exit (0);
}
